RepTracker = LibStub("AceAddon-3.0"):NewAddon("RepTracker", "AceConsole-3.0", "AceEvent-3.0")

local defaults = {
	profile = {
		showInChat = true,
		dynamic = false,
		decrease = false,
		noreset = false,
		ZoneToFaction = {},
	},
}

local options = {
	name = "RepTracker",
	handler = RepTracker,
	type="group",
	args = {
		track = {
			order = 1,
			type = 'input',
			name = 'Current faction tracked:',
			desc = 'Sets which faction is tracked in this zone by default.  <none> to remove the default.',
			get = "GetZone",
			set = "SetZone"
		},
		trackDesc = {
			order = 2,
			type = 'description',
			name = 'Sets which faction is tracked in this zone by default.  <none> to remove the default.',
		},
		toggleHeader = {
			order = 3,
			type = 'header',
			name = 'Toggle Options'
		},
		sticky = {
			order = 6,
			type = 'toggle',
			name = 'Enable Sticky Rep',
			desc = 'Toggles whether the dynamic tracking changes are permanently stored as the default tracked faction.',
			get = "IsSticky",
			set = "ToggleSticky",
			disabled = function() return not RepTracker.db.profile.dynamic end
		},
		stickyDesc = {
			order = 7,
			type = 'description',
			name = 'Toggles whether the dynamic tracking changes are permanently stored as the default tracked faction.',
		},
		dynamic = {
			order = 4,
			type = 'toggle',
			name = 'Enable Dynamic Tracking',
			desc = 'Toggles whether the tracked rep changes to reflect the last faction you gained rep with.',
			get = "IsDynamic",
			set = "ToggleDynamic"
		},
		dynamicDesc = {
			order = 5,
			type = 'description',
			name = 'Toggles whether the tracked rep changes to reflect the last faction you gained rep with.',
		},
		showInChat = {
			order = 8,
			type = 'toggle',
			name = 'Enable Show in Chat',
			desc = 'Toggles the display of RepTracker messages in the chat window.',
			get = "IsShowInChat",
			set = "ToggleShowInChat"
		},
		showInChatDesc = {
			order = 9,
			type = 'description',
			name = 'Toggles the display of the tracking change notification in the chat window.',
		},		
		decrease = {
			order = 10,
			type = 'toggle',
			name = 'Enable Tracking Decreases',
			desc = 'Toggles tracking of reputation decreases.',
			get = "IsDecrease",
			set = "ToggleDecrease",
		},
		decreaseDesc = {
			order = 11,
			type = 'description',
			name = 'Toggles tracking of reputation decreases.'
		},
		noreset = {
			order = 12,
			type = 'toggle',
			name = 'Disable Faction Reset',
			desc = 'Do not reset if no faction found.',
			get = "NoReset",
			set = "ToggleNoReset",
		},
		noresetDesc = {
			order = 13,
			type = 'description',
			name = 'Disables resetting (tracking no faction) if no faction is set or found.'
		},		
		aboutHeader = {
			order = 14,
			type = 'header',
			name = 'About'
		},
		version = {
			order = 15,
			type = 'description',
			name = 'Version: |cFFFF80801.5.3|r',
		},
		author = {
			order = 16,
			type = 'description',
			name = 'by |cFFFF8080InSo(v)nIaX|r',
		},
	},
}

function RepTracker:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RepTrackerDB", defaults, "Default")

	LibStub("AceConfig-3.0"):RegisterOptionsTable("RepTracker", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RepTracker", "RepTracker")
	self:RegisterChatCommand("rt", "ChatCommand")
	self:RegisterChatCommand("RepTracker", "ChatCommand")
	if self.db.profile.showInChat then
		local faction = self:GetZone()
			if faction ~= nil then
			self:Print("Zoned into "..GetZoneText()..", tracking "..faction.." reputation")
			end
	end
end

function RepTracker:OnEnable()
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	if self.db.profile.dynamic then
		self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
	end
end

function RepTracker:OnDisable()
	self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
	self:UnregisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
end

function RepTracker:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToFrame(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(RepTracker, "rt", "RepTracker", input)
    end
end

function RepTracker:IsShowInChat(info)
	return self.db.profile.showInChat
end

function RepTracker:ToggleShowInChat(info, value)
	self.db.profile.showInChat = value
end

function RepTracker:IsDynamic(info)
	return self.db.profile.dynamic
end

function RepTracker:ToggleDynamic(info, value)
	self.db.profile.dynamic = value
	if(self.db.profile.dynamic == false) then
		self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
	else
		self:UnregisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
	end
end

function RepTracker:IsSticky(info)
	return self.db.profile.sticky
end

function RepTracker:ToggleSticky(info, value)
	self.db.profile.sticky = value
end

function RepTracker:IsDecrease(info)
	return self.db.profile.decrease
end

function RepTracker:ToggleDecrease(info, value)
	self.db.profile.decrease = value
end

function RepTracker:NoReset(info)
	return self.db.profile.noreset
end

function RepTracker:ToggleNoReset(info, value)
	self.db.profile.noreset = value
end

function RepTracker:ScanFactions()
	local factions = {}
	local name, isHeader, isCollapsed
	factions["none"] = 0
	for i = 0, GetNumFactions() do
		_, _, _, _, _, _, _, _, isHeader, isCollapsed = GetFactionInfo(i)
		if isHeader and isCollapsed then
			ExpandFactionHeader(i)
		end
	end
	for i = 0, GetNumFactions() do
		name, _, _, _, _, _, _, _, isHeader = GetFactionInfo(i)
		if not isHeader and name ~= nil then
			factions[string.lower(name)] = i
		end
	end
	self.db.profile.factions = factions
end

function RepTracker:ZONE_CHANGED_NEW_AREA()
	self:ScanFactions()
	local faction = self:GetZone()
	if faction == nil then
		if self.db.profile.showInChat then
			self:Print("Zoned into "..GetZoneText()..", no faction tracked")
		end
		self:Print("To set the default tracked faction for this zone, type: /RepTracker track <faction>")
		if not self.db.profile.noreset then
			SetWatchedFactionIndex(0)
		end
	else
		faction = string.lower(faction)
		if(self.db.profile.factions[faction] ~= nil) then
			SetWatchedFactionIndex(self.db.profile.factions[faction])
			if self.db.profile.showInChat then
				self:Print("Zoned into "..GetZoneText()..", tracking "..faction.." reputation")
			end
		else
			self:Print("You have not made contact with that faction yet; tracking none")
			if not self.db.profile.noreset then
				SetWatchedFactionIndex(0)
			end
		end
	end
end

function RepTracker:GetZone()
	return self.db.profile.ZoneToFaction[GetZoneText()]
end

function RepTracker:SetZone(info, value)
	self:ScanFactions()
	if(self.db.profile.factions[string.lower(value)] ~= nil) then
		self.db.profile.ZoneToFaction[GetZoneText()] = string.lower(value)
		SetWatchedFactionIndex(self.db.profile.factions[string.lower(value)])
		if self.db.profile.showInChat then
			self:Print("Now tracking "..value.." reputation")
		end
	else
		self:Print("You have not made contact with that faction")
	end
end

function RepTracker:CHAT_MSG_COMBAT_FACTION_CHANGE()
	local faction

	faction,_ = RTGlobalParser_ParseMessage(arg1, FACTION_STANDING_INCREASED)
	if faction then
		self:changeBar(faction);
		return;
	end

	faction,_ = RTGlobalParser_ParseMessage(arg1, FACTION_STANDING_DECREASED)
	if faction and decrease then
		self:changeBar(faction);
		return;
	end
end

function RepTracker:changeBar(faction)
	local oldName,_,_,_,_ = GetWatchedFactionInfo();
	for i=1, GetNumFactions() do
		local name,_,_,_,_,_,_,_,_,_ = GetFactionInfo(i);
		if (name == faction) then
			if not(name == oldName) then
				SetWatchedFactionIndex(i)
				if (self.db.profile.sticky == false) then
					self:Print("Now tracking "..name.." reputation")
				end
				self.db.profile.lastWatched = i
				if self.db.profile.sticky then
					self:SetZone(oldName, faction)
				end
			end
		end
	end
end